************************************************************************
*****                                                              *****
***** @program name: x1_EnergyFlowAnalysis                         *****
*****                                                              *****
***** @task:   Perform modal energy flow analysis                  *****
*****          for orthonormal mode system                         *****
*****                                                              *****
***** @input: - yX_GM.dat  [process file, containing NEM, ...]     *****
*****                                                              *****
***** @output: z1_TKE.dat (instantaneous values of TKE),           *****
*****          z1_ProductionTKE[_m1].dat (inst. and aver. P),      *****
*****          ...                                                 *****
*****                                                              *****
***** @dependency: requires the execution of following programs:   *****
*****         1) x1_CorrelationMatrix.f                            *****
*****         2) x1_solveFredholm.f                                *****
*****         3) x1_POD.f                                          *****
*****         4) x1GalerkinApproximation.f                         *****
*****            (if shift mode is used)                           *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd Noack                                         *****
*****                                                              *****
***** @see:    Noack & Cordier 2010 CEAT report                    *****
*****                                                              *****
***** @revised: 2012-03-10 Berlin/K"oln/Bonn                       *****
***** @revised: 2012-01-15 Syracuse (binaries in xTools/Y*)        *****
***** @revised: 2011-02-10 Poitiers                                *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @revised: 2010-10-25 Poitiers                                *****
***** @created: 2007-08-05 Berlin                                  *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1EnergyFlowAnalysis
      implicit none
* --- Variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for energy flow analysis
      real*8 fAVal(0:NEMP,0:NFlowsP)        ! a
      real*8 fKVal(0:NEMP,0:NFlowsP)        ! K 
      real*8 fPVal(0:NEMP,0:NFlowsP)        ! P
      real*8 fDVal(0:NEMP,0:NFlowsP)        ! D
      real*8 fCVal(0:NEMP,0:NFlowsP)        ! C
      real*8 fTVal(0:NEMP,0:NFlowsP)        ! T
      real*8 fFVal(0:NEMP,0:NFlowsP)        ! F
* ---
      real*8 fAValSub(0:NEMP,0:NFlowsP)     ! a
      real*8 fKValSub(0:NEMP,0:NFlowsP)     ! K 
      real*8 fPValSub(0:NEMP,0:NFlowsP)     ! P
      real*8 fDValSub(0:NEMP,0:NFlowsP)     ! D
      real*8 fCValSub(0:NEMP,0:NFlowsP)     ! C
      real*8 fTValSub(0:NEMP,0:NFlowsP)     ! T
      real*8 fFValSub(0:NEMP,0:NFlowsP)     ! F
* --- File name
      character*120 fn
      integer       fnLength
* --- Auxiliary variables
      integer iDomain, NDomain         ! for xAMC domains
      integer iMode                    ! counter for expansion modes
      integer iFlow                    ! counter for snapshots
      integer iDummy
      real*8  fDummy
* === Begin main segment ============================================= *
      write(*,*) '***********************************'
      write(*,*) '     Modal energy flow analysis'
      write(*,*) '--- Bernd R. Noack, 2010-11-09 ---'
      write(*,*) '***********************************'
      write(*,*)
      write(*,*)
* --- Read parameters ------------------------------------------------ *
      call readYGA
      call countFlows1(iDummy)
      NFlows = iDummy
      call countDomains(NDomain)
* ---
      write(*,*) 'NFlows=',NFlows
      write(*,*) 'NDomain=',NDomain
      call pressRETURN
* --- Zero all arrays
      call initMEFA(fAVal,fKVal,fPVal,fDVal,fCVal,fTVal,fFVal) 
* --- Loop over domains ---------------------------------------------- *
      do iDomain = 1, NDomain 
* ----- Create keyboard file
        open(10,file='tmp.input',status='UNKNOWN',form='formatted')
        write(10,*) iDomain
        write(10,*)
        write(10,*)
        write(10,*)
        close(10)
* ----- Run computation on subdomain
        call system("./xTools/Y1_EnergyFlowAnalysisSub < tmp.input")
* ----- Zero the subdomain array
        call initMEFA(        fAValSub,fKValSub,fPValSub,fDValSub,
     &                        fCValSub,fTValSub,fFValSub) 
        call readMEFA(iDomain,fAValSub,fKValSub,fPValSub,fDValSub,
     &                        fCValSub,fTValSub,fFValSub) 
        do iMode = 0, NEMP
        do iFlow = 0, NFlowsP
          fAVal(iMode,iFlow)=fAVal(iMode,iFlow)+fAValSub(iMode,iFlow) 
          fKVal(iMode,iFlow)=fKVal(iMode,iFlow)+fKValSub(iMode,iFlow) 
          fPVal(iMode,iFlow)=fPVal(iMode,iFlow)+fPValSub(iMode,iFlow)  
          fDVal(iMode,iFlow)=fDVal(iMode,iFlow)+fDValSub(iMode,iFlow) 
          fCVal(iMode,iFlow)=fCVal(iMode,iFlow)+fCValSub(iMode,iFlow) 
          fTVal(iMode,iFlow)=fTVal(iMode,iFlow)+fTValSub(iMode,iFlow) 
          fFVal(iMode,iFlow)=fFVal(iMode,iFlow)+fFValSub(iMode,iFlow) 
        end do
        end do
* --- End of loop over domains --------------------------------------- *
      end do
* --- Output of the modal energy flow analysis ----------------------- *
      iDomain = -1
      call writeMEFA(iDomain,fAVal,fKVal,fPVal,fDVal,fCVal,fTVal,fFVal)
* === End of main segment ============================================ *
      stop
      end
