************************************************************************
*****                                                              *****
***** @program name: x1_EnergyFlowAnalysisSub                      *****
*****                                                              *****
***** @task:   Perform modal energy flow analysis on subdomains    *****
*****                                                              *****
***** @input: - y[1,2]_GM.dat  [job file, containing NKL, NEM, ...]*****
*****         - modes specified in y1_Flows.dat                    *****
*****                                                              *****
***** @output: z1_TKE.XXXX (instantaneous values of TKE),          *****
*****          z1_ProductionTKE[_m1].XXXX (inst. and aver. P),     *****
*****          ...                                                 *****
*****                                                              *****
***** @dependency: (see x1_EnergyFlowAnalysis.f)                   *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd Noack                                         *****
*****                                                              *****
***** @see:    see x1_EnergyFlowAnalysis.f                         *****
*****                                                              *****
***** @revised: 2012-03-10 Berlin/K"oln/Bonn                       *****
***** @revised: 2010-11-09 Poitiers                                *****
***** @revised: 2010-10-25 Poitiers                                *****
***** @created: 2007-08-05 Berlin                                  *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1EnergyFlowAnalysisSub
      implicit none
* --- Variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- Variables for xAMC data
      include 'xv3X.include'           ! for NXSub, ...
      include 'xv3X_GridSub.include'   ! for xSubG, ...
      include 'xv3X_FlowSub.include'   ! for fFlowSubG ! one snapshot
      include 'xv3X_ModesSub.include'  ! for fModesSubG (modes)
* --- Variables for energy flow analysis
      real*8 fAim (-NAMP:NEMP,1:NFlowsP)    ! a from POD
      real*8 fAVal(0:NEMP,0:NFlowsP)        ! a
      real*8 fKVal(0:NEMP,0:NFlowsP)        ! K 
      real*8 fPVal(0:NEMP,0:NFlowsP)        ! P
      real*8 fDVal(0:NEMP,0:NFlowsP)        ! D
      real*8 fCVal(0:NEMP,0:NFlowsP)        ! C
      real*8 fTVal(0:NEMP,0:NFlowsP)        ! T
      real*8 fFVal(0:NEMP,0:NFlowsP)        ! F
* === fKVal(iMode,iFlow) .... TKE of snapshot iFlow in mode iMode
* === fKVal(0,    iFlow) .... total TKE of snapshot iFlow
* === fKVal(iMode,    0) .... averaged TKE  in mode iMode
* --- Variables for NSE operators
      real*8 fFctInnerProductSub
      real*8 fFctViscousTermSub
      real*8 fFctConvectiveTermSub
      real*8 fFctPressureTermSub
* --- File name
      character*120 fn
      integer       fnLength
* --- Auxiliary variables
      integer iDomain, NDomain         ! for xAMC domains
      integer iMode                    ! counter for expansion modes
      integer iFlow                    ! counter for snapshots
      integer iDummy
      real*8  fDummy
* === Begin main segment ============================================= *
      write(*,*) '***********************************'
      write(*,*) '    Modal energy flow analysis'
      write(*,*) '--- Bernd R. Noack, 2010-11-09 ----'
      write(*,*) '***********************************'
      write(*,*)
      write(*,*)
* --- Read parameters ------------------------------------------------ *
      call readYGA
      call readYGP
      write(*,*) 'Read domain #'
      read (*,*) iDomain
* --- Compute NFlows
      call countFlows1(iDummy) 
      NFlows = iDummy
* --- Zero all arrays
      call initMEFA(fAVal,fKVal,fPVal,fDVal,fCVal,fTVal,fFVal) 
* --- Read grid ------------------------------------------------------ *
      call readGridSub(iDomain)       ! needed for initWeight, etc.
* --- Read modes ----------------------------------------------------- *
      do iMode   = -NAM, NEM
        call readFilenameMode1(iMode,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,
     &                   fModesSubG(1,1,1,1,iMode))
C        call testMode(iMode,fModesSubG(1,1,1,1,iMode))
      end do
* --- Read modes ----------------------------------------------------- *
      do iFlow   = 1, NFlows
        call readFilenameFlow1(iFlow,fn,fnLength) ! 2 >> fnshot2
        call readFlowSub(iDomain,fn,fnLength,fFlowSubG)
* ----- Subtract mean flow
        call addFlowOrig(fFlowSubG,
     &                   fModesSubG(1,1,1,1,0),    -1.0D0)
* ----- Update energy flow analysis -------------------------------- *
        write(*,*) 'Updating the e-flow analysis for iDomain=',
     &              iDomain,'...'
        do iMode = 1, NEM
          fAVal(iMode,iFlow) = fAVal(iMode,iFlow)
     &    + fFctInnerProductSub(fModesSubG(1,1,1,1,iMode), 
     &                         fFlowSubG )
        end do
* --- End of loop over snapshots
      end do

* --- Read grid ------------------------------------------------------ *
      call readGridSub(iDomain)       ! needed for initWeight, etc.
* --- Read modes ----------------------------------------------------- *
      do iMode   = -NAM, NEM
        call readFilenameMode1(iMode,fn,fnLength)
        call readFlowSub(iDomain,fn,fnLength,
     &                   fModesSubG(1,1,1,1,iMode))
C        call testMode(iMode,fModesSubG(1,1,1,1,iMode))
      end do
* --- Read modes ----------------------------------------------------- *
      do iFlow   = 1, NFlows
        call readFilenameFlow1(iFlow,fn,fnLength) ! 2 >> fnshot2
        call readFlowSub(iDomain,fn,fnLength,fFlowSubG)
* ----- Subtract mean flow
        call addFlowOrig(fFlowSubG,
     &                   fModesSubG(1,1,1,1,0),
     &                   -1.0D0)
* ----- Update energy flow analysis ---------------------------------- *
        write(*,*) 'Updating the e-flow analysis for iDomain=',
     &              iDomain,'...'
        do iMode = 1, NEM
* ----- K = (u_i,u')^2 / 2
          fKVal(iMode,iFlow) =   fKVal(iMode,iFlow)
     &    + fFctInnerProductSub(    fModesSubG(1,1,1,1,iMode), 
     &                              fFlowSubG )**2 / 2.
* --- P = C(a_i u_i, u',u_0)
            fPVal(iMode,iFlow) = fPVal(iMode,iFlow)
     &      + fAVal(iMode,iFlow)
     &      * fFctConvectiveTermSub(fModesSubG(1,1,1,1,iMode),
     &                              fFlowSubG,
     &                              fModesSubG(1,1,1,1,0))
* --- C = C(a_i u_i, u_0, u')
            fCVal(iMode,iFlow) = fCVal(iMode,iFlow)
     &      + fAVal(iMode,iFlow)
     &      * fFctConvectiveTermSub(fModesSubG(1,1,1,1,iMode),
     &                              fModesSubG(1,1,1,1,0),
     &                              fFlowSubG)
* --- T = T(a_i u_i, u', u')
            fTVal(iMode,iFlow) = fTVal(iMode,iFlow)
     &      + fAVal(iMode,iFlow)
     &      * fFctConvectiveTermSub(fModesSubG(1,1,1,1,iMode),
     &                              fFlowSubG,
     &                              fFlowSubG)
* --- D = D(a_i u_i, u') / Re
            fDVal(iMode,iFlow) = fDVal(iMode,iFlow)
     &      + fAVal(iMode,iFlow)
     &      * fFctViscousTermSub(   fModesSubG(1,1,1,1,iMode),
     &                              fFlowSubG)/fRe
* --- F = F(a_i u_i, p')
          if (NVarFlowSub.le.NVarGridSub) then
            fFVal(iMode,iFlow) = 0.0 ! dim(x)=dim(u) => no p field!
          else
            fFVal(iMode,iFlow) = fFVal(iMode,iFlow)
     &      + fAVal(iMode,iFlow)
     &      * fFctPressureTermSub(  fModesSubG(1,1,1,1,iMode),
     &                              fFlowSubG(1,1,1,NVarFlowSub))
          end if
* ----- End of loop over modes
        end do
* --- End of loop over snapshots
      end do
* --- Output of the modal energy flow analysis ----------------------- *
      call writeMEFA(iDomain,fAVal,fKVal,fPVal,fDVal,fCVal,fTVal,fFVal)
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Add fFactor x flow field ======================================= *
      subroutine addFlowOrig(fFlowSubG,fFlow2SubG,fFactor)
      implicit none
      include 'xv3X.include'
      include 'xv3X_FlowSub.include'
      include 'xv3X_Flow2Sub.include'
      real*8  fFactor
* --- Auxiliary variables
      integer ix, iy, iz, iVar
* --- Begin of segment ----------------------------------------------- *
      do iVar=1,NVarFlowSub
      do iz  =1,NZSub
      do iy  =1,NYSub
      do ix  =1,NXSub
        fFlowSubG(ix,iy,iz,iVar) = fFlowSubG (ix,iy,iz,iVar)
     &                    + fFlow2SubG(ix,iy,iz,iVar) * fFactor
      end do
      end do
      end do
      end do
* --- End of segment ------------------------------------------------- *
      return
      end
