************************************************************************
*****                                                              *****
***** @program name: x1_eigenproblemDMD2                           *****
*****                                                              *****
***** @task:   Solve DMD eigenproblem                              ***** 
*****          --- this program is grid independent !!!            *****
*****          output = DMD eigenvectors (a_i normalized)          *****
*****          not Ritz eigenvectors which coincide                *****
*****          with Fourier modes for periodic dynamics            *****
*****                                                              *****
***** @input:  - z1_CorrelationMatrix.dat  [correlation matrix]    *****
*****                                                              *****
***** @output: - z1_SpectrumDMD.dat,  [DMD eigenvalues]            *****
*****          - z1_TransformationMatrixDMD.dat,  [transf. matrix] *****
*****          - z1_Amplitudes.dat                                 *****
*****                                                              *****
*****                                                              *****
***** @dependency: requires the execution of following programs:   *****
*****          - x1_CorrelationMatrix.f                            *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd R. Noack                                      *****
*****                                                              *****
***** @see:    x1_CorrelationMatrix.f                              *****
*****                                                              *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-06-19 Princeton                               *****
***** @created: 2006-06-13 Poitiers (begin)                        *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1eigenproblemDMD2
      implicit none

* --- Galerkin system
      include 'x1_eigenproblemDMD2.include'      ! for fnCorrelation ...
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
* --- DMD analysis
      real*8  fMatrixCorr (NFlowsP,NFlowsP)      ! correlation matrix
      real*8  fMatrixC    (NFlowsP,NFlowsP)      ! companion matrix
* ---
      complex*16  cEigVal  (NFlowsP)             ! complex eigenvalues (NFlowsP, not NEMP!)
      real*8      fEigValRe (NFlowsP)            ! real parts of eigenvalues
      real*8      fEigValIm (NFlowsP)            ! imaginary parts of eigenvalues
* ---
      real*8  fMatrixT   (NFlowsP,NFlowsP)       ! transformation matrix T_i^m
* --- integer dummy variables
      integer iEM, iFlow
      integer iDummy, jDummy, kDummy
* --- real dummy variables
      real*8  fDummy
      real*8  fVector    (NFlowsP)
      real*8  fMatrix    (NFlowsP,NFlowsP)
      real*8  fMatrixInv (NFlowsP,NFlowsP)
* --- complex dummy variables
      complex*16 cDummy
      complex*16 cVector (NFlowsP)
      complex*16 cMatrixA(NFlowsP,NFlowsP)
      complex*16 cMatrixB(NFlowsP,NFlowsP)
* --- 
      character*120 fn
      integer       fnLength

* === Begin main segment ============================================= *
      write(*,*) '**************************************************'
      write(*,*) '****        Dynamic Mode Decomposition        ****'
      write(*,*) '****     --- eigenproblem in time domain      ****'
      write(*,*) '**************************************************'
      write(*,*) '------------------- BRN 101107 -------------------'
      write(*,*)

* --- Read parameters ------------------------------------------------ *
      call readYGA
      call readFileSystemIO

* --- Read correlation matrix ---------------------------------------- *
      write(*,*) 'Reading correlation matrix ...'
      call readList2d(fnCorrelation,fnCorrelationLength,fMatrixCorr,
     &                iDummy, NFlows,1,NFlowsP,
     &                jDummy,NFlows,1,NFlowsP,0.0D0) ! 0.0> initialize
***** Note that the snapshot range is determined in readList2d!
***** Hence, the iDummy, jDummy arguments in the list.

* --- Compute companion matrix --------------------------------------- *
      write(*,*) 'Computing u^M = Sum_m=1^M-1 c_m u^m ...'
* --- Copy correlation matrix in "A", since MatInv destroys the argument
      do iDummy  = 1, NFlows
      do jDummy = 1, NFlows
         fMatrix(iDummy,jDummy) = fMatrixCorr(iDummy,jDummy)
      end do
      end do
* --- Compute the inverse of "A" in R^{M-1,M-1}
      call MatInv(fMatrix,fMatrixInv,NFlows-1,NFlowsP)
* --- Compute the "c_i, i=1,...,M-1" values in the companion matrix
* --- c = A^{-1} (C^{*,M}) 
      do iDummy = 1, NFlows-1
        fDummy = 0
        do jDummy = 1, NFlows -1
          fDummy = fDummy + fMatrixInv (iDummy, jDummy)
     &                    * fMatrixCorr(jDummy,NFlows)
        end do
        fVector(iDummy) = fDummy
      end do
* --- Compute M-1 x M-1 companion matrix ----------------------------- *
      write(*,*) 'Assemble companion matrix ...'
      do iDummy = 1, NFlowsP
      do jDummy = 1, NFlowsP
        fMatrixC(iDummy,jDummy) = 0.0
      end do
      end do
      do iDummy = 1, NFlows-2
        fMatrixC(iDummy+1,iDummy) = 1.0
      end do
      do iDummy = 1, NFlows-1
        fMatrixC(iDummy,NFlows-1) = fVector(iDummy)
      end do
* --- Copy for EigVal
      do iDummy = 1, NFlows-1
      do jDummy = 1, NFlows-1
        fMatrix(iDummy,jDummy)=fMatrixC(iDummy,jDummy)
      end do
      end do
* --- 
      call writeList2d(fnCompanion,fnCompanionLength,fMatrixC,
     &                1,NFlows-1,1,NFlowsP,
     &                1,NFlows-1,1,NFlowsP,1.0D0)
   
* --- Compute DMD spectrum ------------------------------------------- *
      write(*,*) 'Computing DMD spectrum ...' 
* --- Copying fMatrixC in fMatrix before it gets destroyed by EigVal
      do iDummy = 1, NFlows-1
      do jDummy = 1, NFlows-1
        fMatrix(iDummy,jDummy)=fMatrixC(iDummy,jDummy)
      end do
      end do
* --- Computing eigenvalues
      call EigVal(fMatrix,fEigValRe,fEigValIm,NFlows-1,NFlowsP)
      call SortEVal(fEigValRe,fEigValIm,NFlows-1)
      do iDummy = 1, NFlows-1
        cEigVal(iDummy) = cmplx(fEigValRe(iDummy),fEigValIm(iDummy))
      end do
*---- Write eigenvalues
      open(10,file=fnSpectrum(1:fnSpectrumLength),status='UNKNOWN',
     &        form='formatted')
      do iDummy =1, NFlows-1
        write(10,'(i5,3(1x,f20.10))')
     &            iDummy, fEigValRe(iDummy), fEigValIm(iDummy),
     &                    abs(cEigVal(iDummy))
      end do
      close(10)
* --- Compute DMD modes ---------------------------------------------- *
      write(*,*) 'Computing DMD modes (in transf. matrix) ...'
      do iDummy =1,NFlowsP
      do jDummy =1,NFlowsP
        fMatrixT(iDummy,jDummy)=0.
      end do 
      end do 
* ---
      do iDummy = 1, NFlows-1
* ----- User info
        write(*,'(i5,a,2f12.5)')
     &          iDummy,'th eigenvector for lambda=', cEigVal(iDummy)
* ----- Complex stability matrix
        do jDummy = 1, NFlows-1
        do kDummy = 1, NFlows-1
          cMatrixA(jDummy,kDummy) = fMatrixC(jDummy,kDummy)
          cMatrixB(jDummy,kDummy) = 0.0
        end do
        end do
* ----- Compute eigenvectors
        call EigVec(cVector,cMatrixA,cMatrixB,
     &              cEigVal(iDummy),NFlows-1,NFlowsP)
* ----- Save result
        if (imag(cEigVal(jDummy)).eq.0) then
          do jDummy = 1, NFlows-1
            fMatrixT(iDummy,jDummy)= real(cVector(jDummy))
          end do
        else if (imag(cEigVal(jDummy)).gt.0) then
          do jDummy = 1, NFlows-1
            fMatrixT(iDummy,jDummy)= real(cVector(jDummy))
          end do
        else
          do jDummy = 1, NFlows-1
            fMatrixT(iDummy,jDummy)= imag(cVector(jDummy))
          end do
        end if
        do jDummy=1,NFlows-1
          write(*,*) iDummy, jDummy, fMatrixT(iDummy,jDummy)
        end do
* --- End of loop over eigenvalues
      end do 
* --- Write Transformation matrix
      call writeList2d(fnTransformation,fnTransformationLength,
     &                 fMatrixT, 
     &                 1,NFlows-1,1,NFlowsP, 
     &                 1,NFlows-1,1,NFlowsP,1.0D0)
* === End of main segment ============================================ *
      stop
      end 


************************************************************************
* === Read file system of I/O ======================================== *
      subroutine readFileSystemIO
      implicit none
      include 'x1_eigenproblemDMD2.include'
      logical bFile
      character*120 Line

* --- Begin of segment ----------------------------------------------- *
      write(*,*) 'Setting default values for IO files ...'
* ..............................123456789012345678901234
      fnCorrelation          = 'z1_CorrelationMatrix.dat'
      fnCorrelationLength    = 24
* ..............................1234567890123456789012
      fnCompanion            = 'z1_CompanionMatrix.dat'
      fnCompanionLength       = 22
* ..............................123456789012345678
      fnSpectrum             = 'z1_SpectrumDMD.dat'
      fnSpectrumLength       = 18
* ..............................123456789012345678901234567890
      fnTransformation       = 'z1_TransformationMatrixDMD.dat'
      fnTransformationLength =  30
* ................................12345678901234567
      fnFourier              = 'z1_Amplitudes.dat'
      fnFourierLength        =  17
* --- End of segment ------------------------------------------------- *
      return
      end
