************************************************************************
*****                                                              *****
***** @program name: x1_eigenproblemPOD                            *****
*****                                                              *****
***** @task:   Solve POD eigenproblem                              ***** 
*****          --- this program is grid independent !!!            *****
*****                                                              *****
***** @input:  - z1_CorrelationMatrix.dat  [correlation matrix]    *****
*****          - y1_GM.dat for "N"                                 *****
*****                                                              *****
***** @output: - z1_SpectrumPOD.dat,  [POD eigenvalues]            *****
*****          - z1_TransformationMatrixPOD.dat,  [transf. matrix] *****
*****          - z1_Amplitudes.dat.   [mode amplitudes]            *****
*****                                                              *****
*****                                                              *****
***** @dependency: requires the execution of following programs:   *****
*****          - x1_CorrelationMatrix.f                            *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd Noack                                         *****
*****                                                              *****
***** @see:    x1_CorrelationMatrix.f                              *****
*****                                                              *****
***** @revised: 2010-11-07 Poitiers                                *****
***** @revised: 2010-06-13 Princeton (new nomenclature)            *****
***** @revised: 2010-04-01 Berlin (new nomenclature)               *****
***** @revised: 2006-07-12 Poitiers (cleaned)                      *****
***** @revised: 2006-06-26 Poitiers (consistency with xGM2c)       *****
***** @revised: 2006-06-22 Poitiers (revised, new xv_gm.f ...)     *****
***** @created: 2006-05-27 Poitiers (begin)                        *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x1eigenproblemPOD
      implicit none

* --- Galerkin system
      include 'x1_eigenproblemPOD.include'    ! for fnCorrelation ...
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...

* --- POD
      real*8  fMatrixC   (NFlowsP,NFlowsP)   ! correlation matrix
      real*8  fMatrixEVec(NFlowsP,NFlowsP)   ! eigenvectors of corrlection matrix
      real*8  fSpectrumC (NFlowsP)           ! eigenvalues (NFlowsP, not NEMP!)
* ---
      real*8  fMatrixT  (NEMP,NFlowsP)       ! transformation matrix T_i^m
      real*8  fAim      (-NAMP:NEMP,NFlowsP) ! Fourier coefficients a_i^m
***** The snapshot index in a_i^m is first
***** since the variables are from outside to inside:
***** {\vec a}^m => (a_i)^m => (a_1,...,a_N)^m

* --- Auxiliary variables
      integer iEM, iFlow
      integer iDummy, iDummy2

* === Begin main segment ============================================= *
      write(*,*) '**************************************************'
      write(*,*) '****             POD Decomposition            ****'
      write(*,*) '****     --- eigenproblem in time domain      ****'
      write(*,*) '**************************************************'
      write(*,*) '------------------- BN 101107 --------------------'
      write(*,*)

* --- Read parameters ------------------------------------------------ *
      call readYGA
      call readFileSystemIO

* --- Read correlation matrix ---------------------------------------- *
      write(*,*) 'Reading correlation matrix ...'
      call readList2d(fnCorrelation,fnCorrelationLength,fMatrixC,
     &                iDummy, NFlows,1,NFlowsP,
     &                iDummy2,NFlows,1,NFlowsP,0.0D0) ! 0.0> initialize
***** Note that the snapshot range is determined in readList2d!
***** Hence, the iDummy, iDummy2 arguments in the list.

* --- Compute Karhunen-Loeve spectrum and eigenvectors --------------- *
      call SpecDecSym(fMatrixC,fMatrixEVec,fSpectrumC,NFlows,NFlowsP)

***** Now fMatrixT contains the normalized eigenvectors in its COLUMNS,
***** and fSpectrumC the eigenvalues
* --- User info
      write(*,*) 'Loading the Fourier coefficients for i<0...'
* --- i < 0
      if (NAM.gt.0) then
        call readAmplitudes1(fAim)
      end if
* --- i = 0
      do iFlow =1, NFlows
        fAim(0,iFlow) = 1.
      end do
* --- Compute mode amplitudes from normalized C eigenvectors
      write(*,*) 'Computing the mode amplitudes for i>0 ...',
     &            NKL,'x',NFlows
* --- i>0
      do iEM   = 1, NKL
      do iFlow = 1, NFlows
         fAim(iEM,iFlow)     = fMatrixEVec(iFlow,iEM)
     &                       * sqrt(fSpectrumC(iEM)*NFlows)
      end do
      end do

* --- Compute transformation matrix from normalized C eigenvectors
      write(*,*) 'Computing the transformation matrix ...',
     &            NKL,'x',NFlows
      do iEM   = 1, NKL
      do iFlow = 1, NFlows
         fMatrixT(iEM,iFlow) = fMatrixEVec(iFlow,iEM)
     &                       / sqrt(NFlows * fSpectrumC(iEM))
      end do
      end do
* --- Write out results ---------------------------------------------- *
      write(*,*) 'Writing out the results in xAMC format ...'
      call writeList1d(fnSpectrum,fnSpectrumLength,
     &                 fSpectrumC,
     &                 1,NKL,1,NFlowsP,1.0D0)
      call writeList2d(fnTransformation,fnTransformationLength,
     &                 fMatrixT, 
     &                 1,NKL, 1,NEMP, 1,NFlows,1,NFlowsP,1.0D0)
      call writeAmplitudes1(fAim)
* === End of main segment ============================================ *
      stop
      end 

************************************************************************
* === Read file system of I/O ======================================== *
      subroutine readFileSystemIO
      implicit none
      include 'x1_eigenproblemPOD.include'
      logical bFile
      character*120 Line
* --- Begin of segment ----------------------------------------------- *
      write(*,*) 'Setting default values for IO files ...'
* ................................123456789012345678901234
      fnCorrelation          = 'z1_CorrelationMatrix.dat'
      fnCorrelationLength    = 24
* ................................123456789012345678
      fnSpectrum             = 'z1_SpectrumPOD.dat'
      fnSpectrumLength       = 18
* ..............................123456789012345678901234567890
      fnTransformation       = 'z1_TransformationMatrixPOD.dat'
      fnTransformationLength =  30
* ................................12345678901234567
      fnFourier              = 'z1_Amplitudes.dat'
      fnFourierLength        =  17
* --- End of segment ------------------------------------------------- *
      return
      end
