************************************************************************
*****                                                              *****
***** @program name: x2_PressureTerm                               *****
*****                                                              *****
***** @task:   Compute linear pressure term representation         *****
*****          for POD Galerkin model                              *****
*****                                                              *****
***** @input: - y1_GM.dat  [process file, containing NEM, ...]     *****
*****         - z0_Data/Flow.* [snapshots]                         *****
*****         - z1_Data/Mode.* [modes]                             *****
*****                                                              *****
***** @output: z1_PressureTerm.0* & dat (u_i, -nabla p^m),         *****
*****          z2_PressureTerm.dat (i,j -> l_ij^p)                 *****
*****          ...                                                 *****
*****                                                              *****
***** @dependency: requires a Galerkin expansion                   *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author: Bernd Noack                                         *****
*****                                                              *****
***** @see:    Noack & Cordier 2012 CEAT report                    *****
*****                                                              *****
***** @revised: 2012-03-11 K"oln -> Poitiers                       *****
***** @created: 2012-03-10 Berlin/K"oln/Bonn                       *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x2PressureTerm
      implicit none
* --- Variables (dimensions) for file system
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv3X.include' ! NVarGridSub, ...
* --- Variables for GP of pressure term
      real*8 fAim(-NAMP:NEMP,  NFlowsP)  ! a_i^m
      real*8 fPVal   (0:NEMP,0:NFlowsP)  ! pressure term (total)
      real*8 fPValSub(0:NEMP,0:NFlowsP)  ! pressure term (subdomain)
* --- ...
      real*8 fVi  (NEMP)                 ! Variance of a_i
      real*8 fLijP(NEMP,0:NEMP)          ! GT of pressure term
* --- File name
      character*120 fn
      integer       fnLength
* --- Auxiliary variables
      integer iDomain, NDomain         ! for xAMC domains
      integer iMode,   jMode           ! counter for expansion modes
      integer iFlow                    ! counter for snapshots
* --- Further auxiliary variables
      integer iDummy,  jDummy
      real*8  fDummy
* === Begin main segment ============================================= *
      write(*,*) '*******************************'
      write(*,*) '      GP of pressure term      '
      write(*,*) '--- Bernd Noack, 2012-03-10 ---'
      write(*,*) '*******************************'
      write(*,*)
* --- Check if pressure fields are included
      if (NVarFlowSub.le.NVarGridSub) then
        write(*,'(a,i3,a,i3)') 'dim(Grid)=',NVarGridSub,
     &                     '; dim(Flow)=',NVarFlowSub
        write(*,*) 'No pressure fields available.'
        stop
      end if
* --- Read parameters ------------------------------------------------ *
      call readYGA
* ---
      call countFlows1(iDummy)
      NFlows = iDummy
      call countDomains(NDomain)
      write(*,*) 'NFlows=',NFlows
      write(*,*) 'NDomain=',NDomain
      call pressRETURN
* --- Zero array
      do iMode=0,NEMP
      do iFlow=0,NFlowsP
        fPVal(iMode,iFlow) = 0
      end do
      end do
* --- Read amplitudes
      call readAmplitudes1(fAim)
* --- Loop over domains ---------------------------------------------- *
      do iDomain = 1, NDomain 
* ----- Create keyboard file
        open(10,file='tmp.input',status='UNKNOWN',form='formatted')
        write(10,*) iDomain
        write(10,*)
        write(10,*)
        write(10,*)
        close(10)
* ----- Run computation on subdomain
        call system("./xTools/Y2_PressureTermSub < tmp.input")
        call readPressureTerm(iDomain,fPValSub)
* ----- Zero the subdomain array
        do iMode = 0, NEMP
        do iFlow = 0, NFlowsP
          fPVal(iMode,iFlow)=fPVal(iMode,iFlow)+fPValSub(iMode,iFlow)  
        end do
        end do
* --- End of loop over domains --------------------------------------- *
      end do
* --- Output of the modal pressure term projection ------------------- *
      iDomain = -1
      call writePressureTerm(iDomain,fPVal)
* --- Computation of the linear pressure term preresentation --------- *
* --- Step 1: Compute Var(a_i)
      do iMode=1,NEM
        fDummy = 0.0
        do iFlow=1,NFlows
          fDummy = fDummy + fAim(iMode,iFlow)**2
        end do
        fVi(iMode) = fDummy/NFlows
        write(*,'(a,i5,a,f12.5)') 'var(a_',iMode,')=',fVi(iMode)
      end do
* --- Step 2: Compute l_ij^P = < (u_i,-nabla p^m) a_j^m >_m / Var(a_i)
      do iMode=1,NEM
      do jMode=0,NEM
        fDummy =  0.0
* ..... constant pressure term
        if (jMode.eq.0) then
          do iFlow=1,NFlows
            fDummy = fDummy + fPVal(iMode,iFlow)
          end do
          fDummy = fDummy / NFlows
* ..... sufficiently energetic modes
        else if (fVi(iMode).gt.0.1*fVi(1)) then
          do iFlow=1,NFlows
            fDummy = fDummy + fPVal(iMode,iFlow)*fAim(jMode,iFlow)
          end do
          fDummy = fDummy / NFlows
          fDummy = fDummy / fVi(iMode)
        end if
* ..... 
        fLijP(iMode,jMode) = fDummy
      end do
      end do
* --- writing out the result
      open(10,file='z2_PressureTerm.dat',
     &        status='UNKNOWN',form='formatted')
      do iMode=1,NEM
      do jMode=0,NEM
        if (fLijP(iMode,jMode).ne.0.) then
          write(10,'(2i5,e20.10)') iMode, jMode, fLijP(iMode,jMode)
        end if
      end do
      end do
      close(10)
* === End of main segment ============================================ *
      stop
      end
