************************************************************************
*****                                                              *****
***** @program name: x3_plotDS                                     *****
***** @task:   visualization of the dynamical systems coefficients *****
***** @input:  y3_DS_Qijk.dat ... parameters of DS                 *****
***** @output: gnu_DS_Ci.dat, ...                                  *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     Noack & Cordier 2012 CEAT report                   *****
***** @revised: 2012-03-08 Poitiers (new data structure)           *****
***** @created: 2011-02-02 Poitiers                                *****
*****                                                              *****
***********************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3plotDS
      implicit none
      include 'xv_DS.include'     ! NAP,... | dimensions of DS
      include 'xv_DS_Flow.include'    ! fFi      | propagator of DS
      integer iOption
* === Begin main segment ============================================= *
      write(*,*) '*************************************'
      write(*,*) '**** Visualize dynamical system *****'
      write(*,*) '*************************************'
      write(*,*) '------------- BN 120307 -------------'
      write(*,*)
* --- Read dynamical system
      call readQijk
* --- Prepare figures
      write(*,*) '-------------------------------------'
      call plotCi
      call plotLij
      call plotQijk
      call writeLinuxPNG
      call writeLaTeXPNG
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Visualize the constant term ==================================== *
      subroutine plotCi
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! fQijk
      integer*8 iDummy
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Visualizing c_i ...'
* --- Write gnuplot file
      open(10,file='gnu_DS_Ci.dat',form='formatted',status='UNKNOWN') 
      do iDummy = 1, NA
        write(10,*) iDummy, fQijk(iDummy,0,0)
      end do
      close(10)
* --- Write gnuplot script
      open(10,file='gnu_DS_Ci.gnuplot',
     &        form='formatted',status='UNKNOWN')
      write(10,'(a)') 'set xlabel "i"'
      write(10,'(a)') 'set ylabel "c_i"'
      write(10,'(a)') 'set grid xtics'
      write(10,'(a)') 'set grid ytics'
      write(10,'(a,i4,a)') 'set xrange [0.5:',NA+0.5,']'
      write(10,'(a)') 'set title "xAMC: c_i"'
      write(10,'(a)') 'set style fill solid'
      write(10,'(a)') 'plot    "gnu_DS_Ci.dat" using 1:2 with boxes'
      write(10,'(a)') 'set out "gnu_DS_Ci.png"'
      write(10,'(a)') 'set terminal png'
      write(10,'(a)') 'replot'
      close(10)
* --- Execute gnuplot script
      call system('gnuplot gnu_DS_Ci.gnuplot')
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Visualize the linear term ====================================== *
      subroutine plotLij
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! fQijk
      integer*8 iDummy, jDummy
      real*8    fDummy, fMin, fMax
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Visualizing l_ij ...'
*     0.5 0.5  1.0     !  
*     0.5 1.5  1.0     !    ---------- 2.5=j
*     0.5 1.5  2.0     !    | 3 || 4 |
*     0.5 2.5  2.0     !    ---------- 1.5=j
*     1.5 0.5  1.0     !    ---------- 1.5=j
*     1.5 1.5  1.0     !    | 1 || 2 |
*     1.5 1.5  2.0     !    ---------- 0.5=j
*     1.5 2.5  2.0     !    |   ||   |___2.5=i
*     1.5 0.5  3.0     !    |   ||_______1.5=i
*     1.5 1.5  3.0     !    |   |________1.5=i
*     1.5 1.5  4.0     !    |____________0.5=i
*     1.5 2.5  4.0     ! 
*     2.5 0.5  3.0     ! Note that the matrix ((1,2),(3,4))
*     2.5 1.5  3.0     ! requires all elements to be specified 2x2 times
*     2.5 1.5  4.0     ! to avoid any averaging over the square.
*     2.5 2.5  4.0
* --- Write gnuplot file
      open(10,file='gnu_DS_Lij.dat',form='formatted',status='UNKNOWN') 
      do iDummy = 1, NA
        do jDummy = 1, NA
          fDummy = fQijk(iDummy,jDummy,0)+fQijk(iDummy,0,jDummy)
          write(10,*) iDummy-0.5, jDummy-0.5, fDummy
          write(10,*) iDummy-0.5, jDummy+0.5, fDummy
        end do
        write(10,*)
        do jDummy = 1, NA
          fDummy = fQijk(iDummy,jDummy,0)+fQijk(iDummy,0,jDummy)
          write(10,*) iDummy+0.5, jDummy-0.5, fDummy
          write(10,*) iDummy+0.5, jDummy+0.5, fDummy
        end do
        write(10,*)
      end do
      close(10)
* --- Determine extrema for plot
      fMin = fQijk(1,1,1)
      fMax = fQijk(1,1,1)
      do iDummy = 1, NA
      do jDummy = 1, NA
         fDummy = fQijk(iDummy,jDummy,0)+fQijk(iDummy,0,jDummy)
         fMax = max(fMax,fDummy)
         fMin = min(fMin,fDummy)
      end do
      end do
* --- Put the zero in the middle, truncate at the end
      fMax = max(abs(fMax),abs(fMin))!/3
      fMin = -fMax
* --- Write gnuplot script
      open(10,file='gnu_DS_Lij.gnuplot',
     &        form='formatted',status='UNKNOWN')
      write(10,'(a)') 'set xlabel "i"'
      write(10,'(a)') 'set ylabel "j"'
      write(10,'(a)') 'set title "xAMC: l_ij"'
      write(10,'(a,i4,a)') 'set xrange [ 0.5:',NA+.5,']'
      write(10,'(a,i4,a)') 'set yrange [ 0.5:',NA+.5,']'
      write(10,'(a)') 'set view map # 2D plot'
C XXX      write(10,'(a)') 'set contour'
C XXX write(10,'(a)') 'set cntrparam levels 10'
      write(10,'(a)') 'set pm3d'
      write(10,'(a,a)') 'set palette model RGB defined ',
     &         '(0 "blue", 0.5 "yellow", 1 "red")'
      write(10,'(a,f12.5,a,f12.5,a)') 
     & 'set cbrange [',fMin,':',fMax,']'
      write(10,'(a)') 'splot "gnu_DS_Lij.dat"'
      write(10,'(a)') 'set grid xtics'
      write(10,'(a)') 'set grid ytics'
      write(10,'(a)') 'set out "gnu_DS_Lij.png"'
      write(10,'(a)') 'set terminal png'
      write(10,'(a)') 'replot'
      close(10)
* --- Execute gnuplot script
      call system('gnuplot gnu_DS_Lij.gnuplot')
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Visualize the quadratic term =================================== *
      subroutine plotQijk
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! fQijk
      integer*8 iDummy, jDummy, kDummy
      real*8    fDummy, fMin,   fMax
      character*120 fn
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Visualizing q_ijk ...'
      do iDummy = 1, NA
* .....  12345678901234567890
* ..... 'gnu_DS_Qijk_0001.dat'
         write(fn,'(a,i4.4)') 'gnu_DS_Qijk_',iDummy
* ------ Write gnuplot file (note that N+1 values are need to plot N intervals
        open(10,file=fn(1:16)//'.dat',form='formatted',status='UNKNOWN') 
* ..... plotLij for the peculiar form of the gnuplot data
        do jDummy = 1, NA
          do kDummy = 1, NA
            fDummy = fQijk(iDummy,jDummy,kDummy)
            write(10,*) jDummy-0.5, kDummy-0.5, fDummy
            write(10,*) jDummy-0.5, kDummy+0.5, fDummy
          end do
          write(10,*)
          do kDummy = 1, NA
            fDummy = fQijk(iDummy,jDummy,kDummy)
            write(10,*) jDummy+0.5, kDummy-0.5, fDummy
            write(10,*) jDummy+0.5, kDummy+0.5, fDummy
          end do
          write(10,*)
        end do
       close(10)
* ----- Determine extrema for plot
        fMin = fQijk(iDummy,1,1)
        fMax = fQijk(iDummy,1,1)
        do jDummy = 1, NA
        do kDummy = 1, NA
           fDummy = fQijk(iDummy,jDummy,kDummy)
           fMax = max(fMax,fDummy)
           fMin = min(fMin,fDummy)
        end do
        end do
* ----- Put the zero in the middle, truncate at the end
        fMax = max(abs(fMax),abs(fMin))!/3
        fMin = -fMax
* ----- Write gnuplot script
        open(10,file=fn(1:16)//'.gnuplot',
     &          form='formatted',status='UNKNOWN')
        write(10,'(a)') 'set xlabel "j"'
        write(10,'(a)') 'set ylabel "k"'
        write(10,'(a,i4,a)') 'set title "q_ijk, i=',iDummy,'"'
        write(10,'(a,f6.1,a)') 'set xrange [ 0:',NA+0.5,']'
        write(10,'(a,f6.1,a)') 'set yrange [ 0:',NA+0.5,']'
        write(10,'(a)') 'set view map'
        write(10,'(a)') 'set pm3d'
        write(10,'(a,a)') 'set palette model RGB defined ',
     &           '(0 "blue", 0.5 "yellow", 1 "red")'
        write(10,'(a,f12.5,a,f12.5,a)') 
     &           'set cbrange [',fMin,':',fMax,']'
        write(10,'(a,i4.4,a)') 'splot "gnu_DS_Qijk_',iDummy,'.dat"'
        write(10,'(a)') 'set grid xtics'
        write(10,'(a)') 'set grid ytics'
        write(10,'(a,i4.4,a)') 'set out "gnu_DS_Qijk_',iDummy,'.png"'
        write(10,'(a)') 'set terminal png'
        write(10,'(a)') 'replot'
        close(10)
* ----- Execute gnuplot script
        open (10,file='tmp.sh',form='formatted',status='UNKNOWN')
        write(10,'(a,i4.4,a)') 'gnuplot gnu_DS_Qijk_',iDummy,'.gnuplot'
        close(10)
        call system('sh tmp.sh')
* --- End of loop over iDummy
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Create Linux script for showing all png ======================== *
      subroutine writeLinuxPNG
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      integer*8 iDummy
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Creating << gnu_DS_CLQ.sh >> ...'
      open(10,file='gnu_DS_CLQ.sh',form='formatted',status='UNKNOWN')
      write(10,'(a)') '$PNGViewer gnu_DS_Ci.png' 
      write(10,'(a)') '$PNGViewer gnu_DS_Lij.png' 
      do iDummy = 1, NA
        write(10,'(a,i4.4,a)') '$PNGViewer gnu_DS_Qijk_',iDummy,'.png' 
      end do
      close(10)
      call system ('chmod u+x gnu_DS_CLQ.sh')
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Create LaTeX for showing all png ======================== *
      subroutine writeLaTeXPNG
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      integer*8 iDummy
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Creating << gnu_DS_CLQ.tex >> ...'
      open(10,file='gnu_DS_CLQ.tex',form='formatted',status='UNKNOWN')
* === NOTE: char(92)='\' should run on all fortran compilers.
* === Different fortran compilers  assume 
* === '\' exclusive or '\\' for the backslash.
      write(10,'(2a)') char(92)//'documentclass[a4paper,twocolumn]',
     &               '{article}'
      write(10,'(a)') char(92)//'usepackage{graphicx}'
      write(10,'(a)') char(92)//'DeclareGraphicsExtensions{png}'
      write(10,'(a)') char(92)//'begin{document}'
* ---
C XXX write(10,'(a)') char(92)//'section{Constant term}'
      write(10,'(a)') char(92)//'begin{figure*}[b]'
      write(10,'(2a)') 
     &         char(92)//'includegraphics[width=140mm,angle=0]',
     &         '{gnu_DS_Ci.png}'
      write(10,'(a,i4,a)') 
     &         char(92)//'caption{Constant term $c_i$ of DS.}'
      write(10,'(a)') char(92)//'end{figure*}'
      write(10,'(a)') char(92)//'clearpage'
* --- linear term
C XXX write(10,'(a)') char(92)//'section{Linear term}'
      write(10,'(a)') char(92)//'begin{figure*}[b]'
      write(10,'(2a)') 
     &         char(92)//'includegraphics[width=140mm,angle=0]',
     &         '{gnu_DS_Lij.png}'
      write(10,'(a,i4,a)') 
     &         char(92)//'caption{Linear term $l_{ij}$ of DS.}'
      write(10,'(a)') char(92)//'end{figure*}'
      write(10,'(a)') char(92)//'clearpage'
* --- quadratic term
C XXX write(10,'(a)') char(92)//'section{Quadratic term}'
      do iDummy=1,NA
      write(10,'(a)') char(92)//'begin{figure}[b]'
      write(10,'(2a,i4.4,a)') 
     & char(92)//'includegraphics[height=77mm,width=100mm,angle=90]',
     &         '{gnu_DS_Qijk_',iDummy,'.png}'
      write(10,'(a,i4,a)')
     &         char(92)//'caption{Quadratic term $q_{ijk}$ for $i=$',
     &         iDummy,'.}'
      write(10,'(a)') char(92)//'end{figure}'
      if (mod(iDummy,2).eq.0) write(10,'(a)') char(92)//'clearpage'
      end do

      write(10,'(a)') char(92)//'end{document}'
      close(10)
* --- Compile LaTeX file
      call system("pdflatex  gnu_DS_CLQ.tex")
* ---
  901 format(1h a)
* --- End of subroutine ---------------------------------------------- *
      return
      end
