************************************************************************
*****                                                              *****
***** @program name: x3_EddyViscosity                              *****
***** @task:   compute modal eddy viscosities                      *****
***** @input:  z3_ModalBalance.dat                                 *****
*****          z2_ViscousTerm.dat                                  *****
***** @output: z3_EddyViscosity.dat                                *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     Noack & Cordier 2010 CEAT report                   *****
***** @revised: 2011-02-05 Poitiers                                *****
***** @created: 2011-02-05 Poitiers                                *****
*****                                                              *****
***********************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3EddyViscosity
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_DS.include' ! NA, NAP
* === NOTE: The input is in 1,2-level notation (NAM, NEM, ...), 
* === the output in 3-level notation
* --- variables for reading the viscous term
      real*8    fLijV(1:NEMP,-NAMP:NEMP)
* --- variables for output
      real*8    fResidual(1:NAP,0:2)
*                         |      |____ 0: lambda_i, 
*                         |      |____ 1: modal Reynolds equation
*                         |      |____ 2: modal TKE equation
*                         |___________ mode
      real*8    fEddyVisc(1:NAP,0:3)
*                         |      |____ 0: l_ii^V, 
*                         |      |____ 1: global nu_T
*                         |      |____ 2: modal nu_T_i
*                         |      |____ 3: filtered nu_T,i = a + i b
*                         |___________ mode
* --- auxiliary variables
      logical bOK
* === Begin main segment ============================================= *
      write(*,*) '*************************************'
      write(*,*) '****** Modal Eddy Viscosity *********'
      write(*,*) '*************************************'
      write(*,*) '------------ BN 110206 --------------'
      write(*,*)
      write(*,*) 'REQUIRES << z3_ModalBalance.dat >>'
      write(*,*) 'FROM     << x3_ModalBalance.f >>'
      write(*,*)
* --- Read amplitudes of full-scale model
      call readViscousTerm(fLijV,bOK)
* --- Read the modal balances
      call readModalBalance(fResidual)
* --- Compute eddy viscosities
      call compEddyViscosity(fLijV,fResidual,fEddyVisc)
* --- Write eddy viscosities
      call writeEddyViscosity(fEddyVisc)
      call writeGNUPlotScript
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Read the model balances from a file ============================ *
      subroutine readModalBalance(fResidual)
      implicit none
      include 'xv_DS.include' ! NA, NAP
      real*8    fResidual(1:NAP,0:2)
      integer*8 iDummy, jDummy
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Reading the modal balances ...'
      NA = 0
      open(10,file='z3_ModalBalance.dat',
     &        form='formatted',status='UNKNOWN')
 101  read(10,*,end=102,err=102) iDummy, 
     &           (fResidual(iDummy,jDummy),jDummy=0,2)
      NA = max(NA,iDummy)
      goto 101
 102  close(10)
* --- user info
      write(*,'(a5,3(2x,a12))') 'i', 'lambda_i','modal RE','modal TKE'
      do iDummy = 1, NA
        write(*,'(i5,3(2x,f12.5))') iDummy, 
     &                        (fResidual(iDummy,jDummy),jDummy=0,2)
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Computing the model eddy viscosities =========================== *
      subroutine compEddyViscosity(fLijV,fResidual,fEddyVisc)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_DS.include' ! NA, NAP
      real*8    fLijV(1:NEMP,-NAMP:NEMP)
      real*8    fResidual(1:NAP,0:2)
      real*8    fEddyVisc(1:NAP,0:3)
      integer*8 iDummy, jDummy
      real*8    fDummy1, fDummy2
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Computing the modal eddy viscosities ...'
* --- l_ii^V (needed for mu_i = l_ii^V nu_T,i)
      do iDummy = 1, NA
        fEddyVisc(iDummy,0) = fLijV(iDummy,iDummy)
      end do
* --- modal eddy viscosity
      do iDummy = 1, NA
        if (fResidual(iDummy,0).gt.0.000001) then
           fDummy1 = fResidual(iDummy,2) ! res_i
           fDummy2 = fResidual(iDummy,0)*fLijV(iDummy,iDummy) ! l_ii^V lambda_i
           fEddyVisc(iDummy,2)=-fDummy1/fDummy2
        end if
      end do
* --- global eddy viscosity
      fDummy1 = 0.0
      fDummy2 = 0.0
      do iDummy = 1, NA
        if (fResidual(iDummy,0).gt.0.000001) then
           fDummy1 = fDummy1 + fResidual(iDummy,2)
           fDummy2 = fDummy2 + fResidual(iDummy,0)*fLijV(iDummy,iDummy)
        end if
      end do
      do iDummy = 1, NA
         fEddyVisc(iDummy,1)=-fDummy1/fDummy2
      end do
* --- filtered eddy viscosity
      do iDummy = 1, NA
         fEddyVisc(iDummy,3)=0.0
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Write the model eddy viscosities in a file ===================== *
      subroutine writeEddyViscosity(fEddyVisc)
      implicit none
      include 'xv_DS.include' ! NA, NAP
      real*8    fEddyVisc(1:NAP,0:2)
      integer*8 iDummy, jDummy
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Writing << z3_EddyViscosity.dat >> ...'
      jDummy = NA
      open(10,file='z3_EddyViscosity.dat',
     &        form='formatted',status='UNKNOWN')
      do iDummy = 1, NA
        write(10,'(i5,4(1x,f20.10))') iDummy, 
     &                        (fEddyVisc(iDummy,jDummy),jDummy=0,3)
      end do
      close(10)
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === write gnuplot script =========================================== *
      subroutine writeGNUPlotScript
      implicit none
      include 'xv_DS.include' ! NA, NAP
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Writing << x3_EddyViscosity.gnuplot >>  ....'
      open(10,file='x3_EddyViscosity.gnuplot',
     &        form='formatted',status='UNKNOWN')

      write(10,'(a)') 'set xlabel "i"'
      write(10,'(a)') 'set ylabel "nu_T,i"'
      write(10,'(a)') 'set grid xtics'
      write(10,'(a)') 'set grid ytics'
      write(10,'(a,i4,a)') 'set xrange [0:',NA+1,']'
      write(10,'(a)') 'set title "xAMC: Modal eddy viscosities"'
      write(10,'(a)') 'set style fill solid'
      write(10,'(2a)') 'plot "z3_EddyViscosity.dat" ',
     &                 'using 1:3 with boxes title "nu_T",\'
      write(10,'(2a)') '     "z3_EddyViscosity.dat" ',
     &                 'using 1:4 with boxes title "nu_T,i"'
      write(10,'(a)') 'set out "gnu_DS_EddyViscosity.png"'
      write(10,'(a)') 'set terminal png'
      write(10,'(a)') 'replot'
      close(10)
* --- End of subroutine ---------------------------------------------- *
      return
      end
