************************************************************************
*****                                                              *****
***** @program name: x3_ModalBalance                               *****
***** @task:   compute residual of modal balances                  *****
****           (modal Reynolds and TKE equation)                   *****
***** @input:  y3_DS_Qijk.dat ... parameters of DS                 *****
*****          z1_Amplitudes .... for statistics                   *****
***** @output: z3_ModalBalance.dat                                 *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     Noack & Cordier 2010 CEAT report                   *****
***** @revised: 2011-02-05 Poitiers                                *****
***** @created: 2011-02-05 Poitiers                                *****
*****                                                              *****
***********************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3ModalBalance
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_DS.include' ! NA, NAP
* --- variables for reading << z1_Amplitudes.dat >>
      real*8    fAim(-NAMP:NEMP,NFlowsP)
* === NOTE: The input is in 1,2-level notation (NAM, NEM, ...), 
* === the output in 3-level notation (
* --- variables for output
      real*8    fResidual(1:NAP,0:2)
*                         |      |____ 0: lambda_i, 
*                         |      |____ 1: modal RE, 2: modal
*                         |___________ mode
*
* === Begin main segment ============================================= *
      write(*,*) '*************************************'
      write(*,*) '****** Modal Balance Equations ******'
      write(*,*) '*************************************'
      write(*,*) '------------ BN 110205 --------------'
      write(*,*)
* --- Read amplitudes of full-scale model
      call readAmplitudes1(fAim)
* --- Read dynamical system
      call readQijk
* --- Prepare figures
      write(*,*) '-------------------------------------'
      call compModalBalance(fAim,fResidual)
      call writeModalBalance(    fResidual)
      call writeGNUPlotScript
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === compute the model balances ===================================== *
      subroutine compModalBalance(fAim,fResidual)
      implicit none
      include 'xv.include'   ! NFlowsP, NEMP, NAMP, NVFP, fRe, ...
      include 'xv_DS.include' ! NA, NAP
      include 'xv_DS_Propagator.include' ! fQijk
      real*8    fAim(-NAMP:NEMP,NFlowsP)
      real*8    fResidual(1:NAP,0:2)
      integer*8 iDummy, jDummy, kDummy, iFlow
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Computing modal balances ...'
      write(*,*) 'N=',NA,'; M=',NFlows
* --- Initialize ----------------------------------------------------- *
      do iDummy = 1, NAP
      do jDummy = 0, 2 
        fResidual(iDummy,jDummy) = 0.0
      end do
      end do
* --- Make sure that "a_0=1"
      do iFlow = 1, NFlows
        fAim(0,iFlow) = 1.0
      end do
* --- Compute balances
      do iFlow = 1, NFlows
        do iDummy = 1, NA
* -------- Update modal fluctuation level (lambda_i) 
          fResidual(iDummy,0) = fResidual(iDummy,0) 
     &                        + fAim(iDummy,iFlow)**2/NFlows
          do jDummy = 0, NA
          do kDummy = 0, NA
* --------- Update modal Reynolds equation
            fResidual(iDummy,1) = fResidual(iDummy,1) 
     &      + fQijk(iDummy,jDummy,kDummy)*fAim(jDummy,iFlow)
     &                                   *fAim(kDummy,iFlow)/NFlows
* --------- Update modal TKE equation
            fResidual(iDummy,2) = fResidual(iDummy,2) 
     &      + fQijk(iDummy,jDummy,kDummy)*fAim(iDummy,iFlow)
     &                                   *fAim(jDummy,iFlow)
     &                                   *fAim(kDummy,iFlow)/NFlows
          end do
          end do
        end do
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Write the model balances in a file ============================= *
      subroutine writeModalBalance(fResidual)
      implicit none
      include 'xv_DS.include' ! NA, NAP
      real*8    fResidual(1:NAP,0:2)
      integer*8 iDummy, jDummy
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Showing the modal balances ...'
* --- user info
      write(*,'(a5,3(2x,a12))') 'i', 'lambda_i','modal RE','modal TKE'
      do iDummy = 1, NA
        write(*,'(i5,3(2x,f12.5))') iDummy, 
     &                        (fResidual(iDummy,jDummy),jDummy=0,2)
      end do
* --- write in file
      write(*,*) 'Writing << z3_ModalBalance.dat >> ...'
      jDummy = NA
      open(10,file='z3_ModalBalance.dat',
     &        form='formatted',status='UNKNOWN')
C XXX NA = jDummy ! QUICK FIX TO A STRANGE PROBLEM: NA=0 after this open!
      do iDummy = 1, NA
        write(10,'(i5,3(1x,f20.10))') iDummy, 
     &                        (fResidual(iDummy,jDummy),jDummy=0,2)
      end do
      close(10)
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === write gnuplot script =========================================== *
      subroutine writeGNUPlotScript
      implicit none
      include 'xv_DS.include' ! NA, NAP
      real*8    fResidual(1:NAP,0:2)
      integer*8 iDummy, jDummy
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Writing << x3_ModalBalance.gnuplot >>  ....'
      open(10,file='x3_ModalBalance.gnuplot',
     &        form='formatted',status='UNKNOWN')

      write(10,'(a)') 'set xlabel "i"'
      write(10,'(a)') 'set ylabel "res_i"'
      write(10,'(a)') 'set grid xtics'
      write(10,'(a)') 'set grid ytics'
      write(10,'(a,i4,a)') 'set xrange [0:',NA+1,']'
      write(10,'(2a)') 'set title "xAMC: ',
     &                 'Residuals of modal Reynolds equation"'
      write(10,'(a)') 'set style fill solid'
      write(10,'(a)') 'plot "z3_ModalBalance.dat" using 1:3 w boxes'
      write(10,'(a)') 'set out "gnu_DS_modalReynolds.png"'
      write(10,'(a)') 'set terminal png'
      write(10,'(a)') 'replot'

      write(10,'(2a)') 'set title "xAMC: ',
     &                'Residuals of modal TKE equation"'
      write(10,'(a)') 'plot "z3_ModalBalance.dat" using 1:4 w boxes'
      write(10,'(a)') 'set out "gnu_DS_modalTKE.png"'
      write(10,'(a)') 'set terminal png'
      write(10,'(a)') 'replot'
      close(10)
* --- End of subroutine ---------------------------------------------- *
      return
      end
