************************************************************************
*****                                                              *****
***** @program name: x3_TurbulenceModel.f                          *****
***** @task:   Integrate modal eddy viscosities in Galerkin system *****
***** @input:  z3_EddyViscosity.dat                                *****
*****          y3_DS_Qijk.dat (also output)                        *****
*****          y3_Modes.dat   (also output)                        *****
*****          y3_State.dat   (also output)                        *****
***** @output: see above                                           *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd Noack                                        *****
***** @see:     Noack & Cordier 2010 CEAT report                   *****
***** @revised: 2011-02-06 Poitiers                                *****
***** @created: 2011-02-06 Poitiers                                *****
*****                                                              *****
***********************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3TurbulenceModel
      implicit none
      include 'xv_DS.include' ! NA, NAP
      include 'xv_DS_State.include'   ! fAi     | state of DS
      real*8  fLiiV(1:NAP)     ! l_ii^V
      real*8  fEddyVisc(1:NAP) ! see x3_EddyViscosity.f
* === Begin main segment ============================================= *
      write(*,*) '*************************************'
      write(*,*) '****** SubgridT urbulenc Model ******'
      write(*,*) '*************************************'
      write(*,*) '------------ BN 110206 --------------'
      write(*,*)
      write(*,*) 'REQUIRES << z3_EddyViscosity.dat >>'
      write(*,*) 'FROM     << x3_EddyViscosity.f >>'
      write(*,*)
* --- Reading DS and eddy viscosities
      call initState(        fAi)
      call readQijk
      call readEddyViscosity(fLiiV,fEddyVisc)
* --- Implement eddy viscositieis in DS
      call compTM(fLiiV,fEddyVisc,fAi)
* --- Write DS
      call writeState(fAi)
      call writeQijk ! This includes q_ijk and z_i
* === End of main segment ============================================ *
      stop
      end

************************************************************************
* === Read the model eddy viscosities in a file ====================== *
      subroutine readEddyViscosity(fLiiV,fEddyVisc)
      implicit none
      include 'xv_DS.include' ! NA, NAP
      real*8    fLiiV    (1:NAP)
      real*8    fEddyVisc(1:NAP)
      real*8    fDummy   (1:NAP,0:3)
      integer*8 iDummy, jDummy
* --- Begin of subroutine -------------------------------------------- *
      write(*,*) 'Reading << z3_EddyViscosity.dat >> ...'
      NA = 0
      open(10,file='z3_EddyViscosity.dat',
     &        form='formatted',status='UNKNOWN')
 101  read(10,*,err=102,end=102) 
     &        iDummy, (fDummy(iDummy,jDummy),jDummy=0,3)
      NA = max(iDummy,NA)
      goto 101
 102  close(10)
* --- user info ------------------------------------------------------ *
      write(*,'(a5,4(2x,a12))') 'i', 'l_ii^V',
     &                          'nu_T','nu_T,i','nu_T,i=a+ib'
      do iDummy = 1, NA
        write(*,'(i5,4(2x,f12.5))') iDummy, 
     &                        (fDummy(iDummy,jDummy),jDummy=0,3)
      end do
* --- user input ----------------------------------------------------- *
      write(*,*) 'Kinds of eddy viscosities as input:'
      write(*,*) '< 1 > ... single eddy viscosity nu_T'
      write(*,*) '< 2 > ... modal  eddy viscosity nu_T,i'
      write(*,*) '< 3 > ... eddy viscosity distribution nu_T,i=a+ib'
      read (*,*) jDummy
      do iDummy = 1, NA
        fLiiV(iDummy)     = fDummy(iDummy,0)
        fEddyVisc(iDummy) = fDummy(iDummy,jDummy)
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Add eddy viscosities in DS (several options) =================== *
      subroutine compTM(fLiiV,fEddyVisc,fAi)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi
      real*8    fLiiV    (1:NAP)
      real*8    fEddyVisc(1:NAP)
      integer*8 iOption
* --- Begin of subroutine -------------------------------------------- *
* --- User info
      write(*,*) 'Implementations of eddy viscosities in the DS:'
      write(*,*) '< 1 > ... constant eddy viscosities in l_ij'
      write(*,*) '< 2 > ... eddy viscosities as new constant state'
      write(*,*) '< 3 > ... eddy viscosities as K-dependent state'
      write(*,*) '< 4 > ... eddy viscosities as dynamic state',
     &           ' based on K'
      write(*,*) 'NOTE: Adaption based on K_i is not included yet.'
      read(*,*) iOption
      if (iOption.eq.1) call compTM1(fLiiV,fEddyVisc)
      if (iOption.eq.2) call compTM2(fLiiV,fEddyVisc,fAi)
      if (iOption.eq.3) call compTM3(fLiiV,fEddyVisc,fAi)
      if (iOption.eq.4) call compTM4(fLiiV,fEddyVisc,fAi)
      if (iOption.lt.1.or.iOption.gt.4) then
        write(*,*) 'No valid option chosen!'
        stop
      end if
      write(*,*)'----------------------------------------'
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Add constant eddy viscosities in original DS =================== *
      subroutine compTM1(fLiiV,fEddyVisc)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_Propagator.include' ! fQijk
      real*8    fLiiV    (1:NAP)
      real*8    fEddyVisc(1:NAP)
      integer*8 iDummy
      real*8    fDummy
* --- Begin of subroutine -------------------------------------------- *
* --- User info
      write(*,*) 'Implement turbulence model #1 ...'
      write(*,*) 'q_ii0 <- q_ii0 + nu_T,i l_ii^V'
* --- Change DS
      write(*,*) 'Protocol of changes:'
      write(*,'(3a5,3a12)') 'i','j','k',
     &        'q_ijk^orig', 'Delta_q_ijk','q_ijk'
      do iDummy = 1, NA
        write(*,'(3i5,3f12.5)') iDummy, iDummy, 0,
     &          fQijk(iDummy,iDummy,0),fEddyVisc(iDummy)*fLiiV(iDummy),
     &          fQijk(iDummy,iDummy,0)+fEddyVisc(iDummy)*fLiiV(iDummy)
        fQijk(iDummy,iDummy,0) = fQijk(iDummy,iDummy,0)
     &                         + fEddyVisc(iDummy)*fLiiV(iDummy)
      end do
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Add eddy viscosities as new constant state in the DS =========== *
      subroutine compTM2(fLiiV,fEddyVisc,fAi)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi
      include 'xv_DS_Propagator.include' ! fQijk
      real*8    fLiiV    (1:NAP)
      real*8    fEddyVisc(1:NAP)
      integer*8 iDummy
      real*8    fDummy
* --- Begin of subroutine -------------------------------------------- *
* --- User info
      write(*,*) 'Implement turbulence model #2 ...'
      write(*,*) 'q_ii{N+1} = l_ii^V nu_T,i'
      write(*,*) 'a_{N+1}   = 1'
* --- Change DS
      write(*,*) 'New DS coefficients:'
      write(*,'(3a5,a12)') 'i','j','k','q_ijk'
* --- Include a_{N+1} in q_ijk as (potential) correction
      do iDummy = 1, NA
        fQijk(iDummy,iDummy,NA+1) = fLiiV(iDummy)*fEddyVisc(iDummy)
        write(*,'(3i5,3f12.5)') 
     &            iDummy, iDummy, NA+1,fQijk(iDummy,iDummy,NA+1)
      end do
* ---
      NA     = NA+1
      iDummy = NA
      fQijk(iDummy,0,0) = 1.0
      write(*,'(3i5,3f12.5)') 
     &            iDummy,0,0,fQijk(iDummy,0,0)
* ---
      fAi(iDummy)   = 1.
      iMode(iDummy) = -1
      write(*,*) 'New states:'
      write(*,'(1a5,a12,a5)') 'i','a_i','z_i'
      write(*,'(1i5,f12.5,1i5)') NA,fAi(iDummy),iMode(iDummy)
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Add eddy viscosities as new K-dependent state in the DS ======== *
      subroutine compTM3(fLiiV,fEddyVisc,fAi)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi
      include 'xv_DS_Propagator.include' ! fQijk
      real*8    fLiiV    (1:NAP)
      real*8    fEddyVisc(1:NAP)
      integer*8 iDummy
      real*8    fDummy, fNorm
* --- Begin of subroutine -------------------------------------------- *
* --- User info
      write(*,*) 'Implement turbulence model #3 ...'
      write(*,*) 'q_ii{N+1} = l_ii^V nu_T,i'
      write(*,*) 'a_{N+1}   = sqrt{K/K_infty}'
* ---
      write(*,*) 'Computing K_infty from << z1_SpectrumPOD.dat >> ...'
      fNorm = 0.0
      open(10,file='z1_SpectrumPOD.dat',form='formatted',status='old')
  101 read(10,*,end=102,err=102) iDummy, fDummy
      fNorm = fNorm + fDummy
      goto 101
  102 close(10)
      write(*,'(a,f12.5)') 'K_infty=',fNorm/2
* --- Change DS ----------------------------------------------------- *
      write(*,*) 'New DS coefficients:'
      write(*,'(3a5,a12)') 'i','j','k','q_ijk'
* --- Include a_{N+1} in q_ijk as (potential) correction
      do iDummy = 1, NA
        fQijk(iDummy,iDummy,NA+1) = fLiiV(iDummy)*fEddyVisc(iDummy)
        write(*,'(3i5,3f12.5)') 
     &            iDummy, iDummy, NA+1,fQijk(iDummy,iDummy,NA+1)
      end do
* --- K equation
      write(*,*) 'New K equation:'
      do iDummy = 1, NA
        fQijk(NA+1,iDummy,iDummy) = 1.0/fNorm
        write(*,'(3i5,3f12.5)') 
     &            NA+1,iDummy,iDummy,fQijk(NA+1,iDummy,iDummy)
      end do
* ---
      NA     = NA+1
      iDummy = NA
      fAi(iDummy)   = 1.
      iMode(iDummy) = -1002 ! This number indicate sqrt{}
      write(*,*) 'New state:'
      write(*,'(1a5,a12,a8)') 'i','a_i','z_i'
      write(*,'(1i5,f12.5,1i8)') NA,fAi(iDummy),iMode(iDummy)
      write(*,'(3a5,a12,a8)') 'i','j','k','q_ijk'
      write(*,'(3i5,3f12.5)') 
     &            iDummy,0,0,fQijk(iDummy,0,0)
* --- End of subroutine ---------------------------------------------- *
      return
      end

************************************************************************
* === Add eddy viscosities as new dynamic state in the DS ============ *
      subroutine compTM4(fLiiV,fEddyVisc,fAi)
      implicit none
      include 'xv_DS.include'        ! NAP,... | dimensions of DS
      include 'xv_DS_State.include'      ! fAi
      include 'xv_DS_Propagator.include' ! fQijk
      real*8    fLiiV    (1:NAP)
      real*8    fEddyVisc(1:NAP)
      integer*8 iDummy
      real*8    fDummy, fNorm
* --- Begin of subroutine -------------------------------------------- *
* --- User info
      write(*,*) 'Implement turbulence model #4 ...'
      write(*,*) 'q_ii{N+1} = l_ii^V nu_T,i'
      write(*,*) 'da_{N+1}/dt = 0.001 (a_{N+2}-1)'
      write(*,*) 'a_{N+2}   = sqrt{K/K_infty}'
* ---
      write(*,*) 'Computing K_infty from << z1_SpectrumPOD.dat >> ...'
      fNorm = 0.0
      open(10,file='z1_SpectrumPOD.dat',form='formatted',status='old')
  101 read(10,*,end=102,err=102) iDummy, fDummy
      fNorm = fNorm + fDummy
      goto 101
  102 close(10)
      write(*,'(a,f12.5)') 'K_infty=',fNorm/2
* --- Change DS ----------------------------------------------------- *
      write(*,*) 'New DS coefficients / states:'
      write(*,'(3a5,2a12,a10)') 'i','j','k','q_ijk','a_i','z_i'
* --- Include a_{N+1} in q_ijk as (potential) correction
      do iDummy = 1, NA
        fQijk(iDummy,iDummy,NA+1) = fLiiV(iDummy)*fEddyVisc(iDummy)
        write(*,'(3i5,2f12.5,i10)') 
     &            iDummy, iDummy, NA+1,fQijk(iDummy,iDummy,NA+1),
     &                                 fAi  (iDummy),iMode(iDummy)
      end do
* --- K equation
      write(*,*) 'New adaption equation:'
      fAi(NA+1)   = 1.0
      iMode(NA+1) = NA+1
      fQijk(NA+1,   0,0) = -0.001
      fQijk(NA+1,NA+2,0) =  0.001
      write(*,'(3i5,2f12.5,i10)') 
     &            NA+1,0,0,fQijk(NA+1,0,0),fAi(NA+1),iMode(NA+1)
      write(*,'(3i5,2f12.5,i10)') 
     &            NA+1,NA+2,fQijk(NA+1,iDummy,iDummy)
* --- K equation
      write(*,*) 'New K equation:'
      fAi(NA+2)   =  1.0
      iMode(NA+2) = -1002
      do iDummy = 1, NA
        fQijk(NA+2,iDummy,iDummy) = 1.0/fNorm
        write(*,'(3i5,2f12.5,i10)') 
     &            NA+2,iDummy,iDummy,fQijk(NA+2,iDummy,iDummy),
     &                               fAi(NA+2),iMode(NA+2)
      end do
      NA = NA+2
* --- End of subroutine ---------------------------------------------- *
      return
      end
