************************************************************************
*****                                                              *****
***** @segment name: xs_FTT_comp.f                                 *****
***** @task: routines for coputing the FTT energy flow             *****
*****        --- always for a constant-linear-quadratic flow       *****
***** @see:  res090905                                             *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012

***********************************************************************
* === enforce partial thermal equilibrium =========================== *
      subroutine enforcePTE(fEi)
      implicit none
      include 'xv_DS.include'     ! NAP,...  | dimensions of DS
      include 'xv_FTT_Mode.include'   ! iModeFTT,...| mode scheme
      real*8    fEi(NAP)              ! E_i      | state of FTT
      real*8    fEg(NAP)              ! E_g      | group energy
      integer*8 NEg(NAP)              ! population of group 
      real*8    fDummy
      integer*8 iDummy, jDummy
* --- begin of subroutine ------------------------------------------- *
C XXX write(*,*) 'Enforcing partial thermal equilibrium in E_i ...'
* XXX No noisy user info, since this routine is called many times.
* --- initializing
      do iDummy = 1, NA
          fEg(iDummy) = 0
          NEg(iDummy) = 0
      end do
* --- group balance
      do iDummy = 1, NA
        jDummy = iModeFTT(iDummy)
* ----- consider only C modes
        if (jDummy.ge.1) then 
          if (jDummy.le.NA) then
            fEg(jDummy) = fEg(jDummy) + fEi(iDummy)
            NEg(jDummy) = NEg(jDummy) + 1
          else
            stop "Error in enforcePTE: iModeFTT > NA!"
          end if
        end if
      end do
* --- enforce partial thermal equilibrium according
* --- Here: Ignore specified fractions in fModeFTT.
      do iDummy = 1, NA
        jDummy = iModeFTT(iDummy)
* ----- consider only C modes
        if (jDummy.ge.1) then
          fEi(iDummy) = fEg(jDummy) / NEg(jDummy)
C DEB     write(*,'(i5,f12.5,i5,f12.5,i5)')
C DEB&            iDummy, fEi(iDummy), jDummy,fEg(jDummy),NEg(jDummy)
        end if
      end do
C DEB read(*,*)
* --- end of subroutine --------------------------------------------- *
      return
      end

***********************************************************************
* === compute Q_i and T_i =========================================== *
      subroutine compQTi(fMi,fEi,fQi,fTi)
      implicit none
      include 'xv_DS.include'         ! NAP,... | dimensions of DS
      real*8    fMi(NAP), fEi(NAP)        ! fMi, E_i | state of FTT 
      real*8    fQi(NAP), fTi(NAP)        ! Q_i, T_i | energy flow of FTT
      real*8    fFctQi, fFctTi
      real*8    fDummy
      integer*8 iDummy
* --- begin of subroutine ------------------------------------------- *
      write(*,*) 'Computing Q_i and T_i ...'
* --- energy flow balance
      do iDummy = 1, NA
          fQi(iDummy) = fFctQi(iDummy,fMi,fEi)
          fTi(iDummy) = fFctTi(iDummy,    fEi)
      end do
* --- user output
* --- end of subroutine --------------------------------------------- *
      return
      end
      

***********************************************************************
* === rescale T_i for energy flow consistency ======================= *
      subroutine rescaleTi(fQi,fTi)
      implicit none
      include 'xv_DS.include'         ! NAP,... | dimensions of DS
      include 'xv_FTT_Propagator.include' ! fAlpha..| for rescaling 
      real*8    fQi(NAP), fTi(NAP)        ! Q_i, T_i
      real*8    fQiDonor, fTiDonor
      real*8    fQiSum, fTiSum
      integer*8 iDummy
      real*8    fRatio
* --- begin of subroutine ------------------------------------------- *
      write(*,*) 'Rescaling T_i and alpha ...'
* --- energy flow balance
      fQiDonor = 0 ! needed for energy balance among donor modes
      fTiDonor = 0 ! dto.
      fQiSum   = 0 ! needed for total energy flow consistency (=!=0)
      fTiSum   = 0 ! dto.
      do iDummy = 1, NA
        if (fQi(iDummy).gt.0) then
          fQiDonor = fQiDonor + fQi(iDummy)
          fTiDonor = fTiDonor + fTi(iDummy)
        end if
          fQiSum   = fQiSum   + fQi(iDummy)
          fTiSum   = fTiSum   + fTi(iDummy)
      end do
* --- rescaling ratio
      if (fQiDonor.gt.0.and.fTiDonor.lt.0) then
        fRatio = -fQiDonor/fTiDonor
      else
        stop "Error in rescalingTi: e-flow consistency not possible."
      end if
* --- rescale alpha and T_i
      fAlpha = fRatio * fAlpha
      do iDummy = 1, NA
        fTi(iDummy) = fRatio*fTi(iDummy)
      end do
* --- user output
      write(*,'(4a12)') 'aggregate', 'Q_i', 'T_i'
      write(*,'(4a12)') '------------','------------',
     &                  '------------','------------'
      write(*,'(a12,3f12.5)') 'donors',
     &                         fQiDonor,fTiDonor,fQiDonor+fTiDonor
      write(*,'(a12,3f12.5)') 'sum',
     &                         fQiSum,  fTiSum,  fQiSum  +fTiSum
      write(*,*) 'fRatio=', fRatio
* --- end of subroutine --------------------------------------------- *
      return
      end
      

***********************************************************************
* === compute FTT prediction for Q_i ================================ *
      real*8 function fFctQi(iDummy,fMi,fEi)
      implicit none
      include 'xv_DS.include'         ! NAP,.... | dimensions of DS
      include 'xv_FTT_Propagator.include' ! fXijk .. | parameters of FTT
      real*8    fMi(NAP), fEi(NAP)        ! m_i, E_i | state of FTT
      integer*8 iDummy, jDummy
      real*8    fDummy
* --- begin of function --------------------------------------------- *
      fDummy = fXijk(iDummy,0,0)
* --- loop for mean-field corrections
      do jDummy = 1, NA
        fDummy = fDummy + fXijk(iDummy,jDummy,0)*fMi(jDummy)
      end do
* --- rescaling with fixed fAlpha
      fFctQi = fDummy * fEi(iDummy)
* --- end of function ----------------------------------------------- *
      return
      end

***********************************************************************
* === redistribution factor of Tijk ================================= *
      real*8 function fFctTi(iDummy,fEi)
* --- This is a function for individual T_i, 
* --- since A and B modes require no T_i computation.
* --- Moreover, the triadic interactions with A modes vanish
* --- due to the vanishing energy.
* --- The triadic interactions with B modes
* --- must be computed and require estimated energy levels.
      implicit none
      include 'xv_DS.include'         ! NAP,... | dimensions of DS
      include 'xv_FTT_Propagator.include' ! fXijk . | 
      real*8    fEi(NAP)
      integer*8 iDummy, jDummy, kDummy
      real*8    fFctFactorTijk, fDummy
* --- begin of function --------------------------------------------- *
      fDummy = 0
* --- loop over all triadic interactions
      do jDummy = 1, NA
      do kDummy = 1, NA
        fDummy = fDummy + fXijk(iDummy,jDummy,kDummy)
     &                  * fFctFactorTijk(fEi(iDummy),
     &                                  fEi(jDummy),
     &                                  fEi(kDummy))
      end do
      end do
* --- rescaling with fixed fAlpha
      fFctTi = fAlpha * fDummy
* --- end of function ----------------------------------------------- *
      return
      end

***********************************************************************
* === redistribution factor of Tijk ================================= *
      real*8 function fFctFactorTijk(fEi,fEj,fEk)
      implicit none
      real*8 fEi, fEj, fEk, fDummy, fEpsilon
      parameter (fEpsilon = 1E-6)
* --- begin of function --------------------------------------------- *
      fDummy  = (fEi+fEj+fEk)/3
      fDummy  = 1 - fEi/max(fDummy,fEpsilon)
      fDummy  = sqrt(fEi*fEj*fEk)*fDummy
      fFctFactorTijk = fDummy
* --- end of function ----------------------------------------------- *
      return
      end
