************************************************************************
*****                                                              *****
***** @segment name: xs_ioList1d.f                                 *****
*****                                                              *****
***** @task: read, write and analyse files of the Galerkin world   *****
***** The subroutines manipulate 1d lists.                         *****
***** Examples are vectors like <a_i>, <K_i>, <P_i>, ...           ***** 
*****                                                              *****
***** @see: header of xGMshow.f                                    *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************

************************************************************************ 
* === read 1d list file ============================================== *
      subroutine readList1d(fn, fnLength,        ! input: file name
     &           fVal, iMin, iMax, iMinP, iMaxP, ! output: 1d array
     &           fFactor)                        ! input: scaling factor
      implicit none
      character*120  fn               ! file name of 1d list
      integer        fnLength         ! length of file name
      integer iMin,  iMax             ! actual   index range of list
      integer iMinP, iMaxP            ! physical index range of list
      real*8  fVal(iMinP:iMaxP)       ! 1d list
      real*8  fFactor, ffFactor       ! factor with wish input is scaled
      integer i,     NVal             ! counters for 1d list
      real*8  ffVal, fValMin, fValMax ! ... for statistics of 1d list
      character*120  Line             ! intermediate input file
      logical lastElement             ! boolean variable
* --- begin segment -------------------------------------------------- *
      write(*,'(3a)') 'Reading a 1D list from ',fn(1:fnLength),'...'

* --- Initialize the list -------------------------------------------- *
      write(*,*) 'Initializing the list ...'
      do i = iMinP, iMaxP
         fVal(i) = 0.0
      end do
      ffFactor = 1.0

      NVal    = 0
      iMin    = iMaxP+1
      iMax    = iMinP-1
      fValMin =  1.E37
      fValMax = -1.E37

* --- open the file [iUnit=10: job file, =11..3: 1d,2d,3d file] ------ *
      write(*,*) 'Reading ',fn(1:fnLength),'...'
      open(11,file  = fn(1:fnLength),
     &        status= 'OLD',
     &        form  = 'formatted')

* --- overwrite contents of line [necessary for last element]
* --- and reading another line of the file
  101 lastElement=.true.
      Line='                                       '//
     &     '                                       '
      read(11,'(a)',end=201) Line
      if (Line(1:1).eq.'[') goto 101   ! skip comments
      lastElement=.false.

* --- add element in list and update statistics
  201 continue
      if (NVal.le.10) write(*,'(a)') Line
      if (NVal.eq.11) write(*,*) '...'
      if (.not.lastElement.or.index(Line,'.').gt.0) then
        NVal = NVal + 1
        read(Line,*) i, ffVal
        if (i.lt.iMinP.or.i.gt.iMaxP) stop 'readList1d: dimension error'
        iMin    = min(iMin,i)
        iMax    = max(iMax,i)
        fValMin = min(fValMin,ffVal)
        fValMax = max(fValMax,ffVal)
        fVal(i) = fVal(i) + ffFactor*ffVal
      end if
      if (.not.lastElement) goto 101

* --- terminate reading from file ------------------------------------ *
      close(11)
      write(*,*) '------------------------------------------'
      write(*,'(3a5,2a14)')   'NVal','iMin','iMax','fMin','fMax'
      write(*,'(3i6,2x,f12.5,2x,f12.5)') 
     &                        NVal, iMin, iMax, fValMin, fValMax
      write(*,*) '------------------------------------------'
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************  
* === write 1d list file ============================================= *
      subroutine writeList1d(fn, fnLength,
     &           fVal, iMin, iMax, iMinP, iMaxP,
     &           fFactor)
      implicit none
      character*120  fn                 ! file name of 1d list
      integer        fnLength           ! length of file name
      integer iMin,  iMax, iMinP, iMaxP ! actual & physical index range
      real*8  fVal(iMinP:iMaxP)         ! 1d list
      real*8  fFactor                   ! factor with wish input is scaled
      integer i, NVal                   ! counters for 1d list

* --- begin segment -------------------------------------------------- *
      write(*,'(3a)') 'Writing a 1D list in ',fn(1:fnLength),'...'

* --- open the file -------------------------------------------------- *
      open(11,file  = fn(1:fnLength),
     &        status= 'UNKNOWN',
     &        form  = 'formatted')

* --- write all nonvanishing elements of specified range in the file
      NVal = 0
      do i=iMin, iMax
        if (fVal(i).ne.0.0) then
          NVal = NVal + 1
          write(11,'(i5,1x,f20.10)') i,fVal(i)
        end if
      end do

* --- terminate writing in the file ---------------------------------- *
      close(11)
      write(*,'(i6,a)')  NVal, ' list elements written in file.'

* --- end segment ---------------------------------------------------- *
      return
      end
