************************************************************************
*****                                                              *****
***** @segment name: xs_ioList2d.f                                 *****
*****                                                              *****
***** @task: read, write and analyse files of the Galerkin world   *****
***** The subroutines manipulate 2d lists.                         *****
***** Examples are modal time histories, e.g. <a_i^m>, <K_i^m>,    ***** 
***** the linear term of the GS l_ij, etc.                         *****
*****                                                              *****
***** @see: header of xGMshow.f                                    *****
***** @see: xs_ioList1d.f for comments                             *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
************************************************************************

************************************************************************ 
* === read 2d list file ============================================== *
      subroutine readList2d(fn, fnLength,        ! input: file name
     &           fVal, iMin, iMax, iMinP, iMaxP, ! output: 2d array
     &                 jMin, jMax, jMinP, jMaxP,
     &           fFactor)                        ! input: scaling factor
      implicit none
      character*120  fn                  ! file name of 2d list
      integer        fnLength            ! length of file name
      integer iMin,  iMax, iMinP, iMaxP  ! actual & physical index range
      integer jMin,  jMax, jMinP, jMaxP
      real*8  fVal(iMinP:iMaxP,jMinP:jMaxP)      ! 2d list
      real*8  fFactor, ffFactor       ! factor with wish input is scaled
      integer i, j,     NVal          ! counters for 2d list
      real*8  ffVal, fValMin, fValMax ! ... for statistics of 2d list
      character*120  Line             ! intermediate input file
      logical lastElement             ! boolean variable

* --- begin segment -------------------------------------------------- *
      write(*,'(3a)') 'Reading a 2d list from ',fn(1:fnLength),'...'

* --- Initialize the list -------------------------------------------- *
      write(*,*) 'Initializing the list ...'
      do j = jMinP, jMaxP
      do i = iMinP, iMaxP
         fVal(i,j) = 0.0
      end do
      end do
      ffFactor = 1.0

      NVal    = 0
      iMin    = iMaxP+1
      iMax    = iMinP-1
      jMin    = jMaxP+1
      jMax    = jMinP-1
      fValMin =  1.E37
      fValMax = -1.E37

* --- open the file [iUnit=10: job file, =11..3: 1d,2d,3d file] ------ *
      write(*,*) 'Reading ',fn(1:fnLength),'...'
      open(12,file  = fn(1:fnLength),
     &        status= 'OLD',
     &        form  = 'formatted')

* --- overwrite contents of line [necessary for last element]
* --- and reading another line of the file
  101 lastElement=.true.
      Line='                                       '//
     &     '                                       '
      read(12,'(a)',end=201) Line
      if (Line(1:1).eq.'[') goto 101   ! skip comments
      lastElement=.false.

* --- add element in list and update statistics
  201 continue
      if (NVal.le.10) write(*,'(a)') Line
      if (NVal.eq.11) write(*,*) '...'
      if (.not.lastElement.or.index(Line,'.').gt.0) then
        NVal = NVal + 1
        read(Line,*) i, j, ffVal
        if (i.lt.iMinP.or.j.gt.jMaxP.or.
     &      j.lt.jMinP.or.j.gt.jMaxP) then
          if (i.lt.iMinP) write(*,*) 'Error: i<iMinP:',i,iMinP
          if (i.lt.iMaxP) write(*,*) 'Error: i>iMaxP:',i,iMaxP
          if (j.lt.jMinP) write(*,*) 'Error: j<jMinP:',j,jMinP
          if (j.lt.jMaxP) write(*,*) 'Error: j>jMaxP:',j,jMaxP
          stop 'readList2d: dimension error'
        end if
        iMin      = min(iMin,i)
        iMax      = max(iMax,i)
        jMin      = min(jMin,j)
        jMax      = max(jMax,j)
        fValMin   = min(fValMin,ffVal)
        fValMax   = max(fValMax,ffVal)
        fVal(i,j) = fVal(i,j) + ffFactor*ffVal
      end if
      if (.not.lastElement) goto 101

* --- terminate reading from file ------------------------------------ *
      close(12)
      write(*,*) '----------------------------------------------------',
     &           '--------'
      write(*,'(a8,4a6,2a14)')   
     &        'NVal','iMin','iMax','jMin','jMax','fMin','fMax'
      write(*,'(i8,4i6,2x,f12.5,2x,f12.5)') 
     &         NVal,  iMin,  iMax,  jMin,  jMax,  fValMin, fValMax
      write(*,*) '----------------------------------------------------',
     &           '--------'
* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************  
* === write 2d list file ============================================= *
      subroutine writeList2d(fn, fnLength,
     &           fVal, iMin, iMax, iMinP, iMaxP,
     &                 jMin, jMax, jMinP, jMaxP,
     &           fFactor)
      implicit none
      character*120 fn                ! file name of 2d list
      integer       fnLength          ! length of file name
      integer iMin,  iMax, iMinP, iMaxP  ! actual & physical index range
      integer jMin,  jMax, jMinP, jMaxP
      real*8  fVal(iMinP:iMaxP,jMinP:jMaxP)      ! 2d list
      real*8  fFactor, ffFactor       ! factor with wish input is scaled
      integer i, j,     NVal          ! counters for 2d list

* --- begin segment -------------------------------------------------- *
      write(*,'(3a)') 'Writing a 2d list in ',fn(1:fnLength),'...'

* --- open the file -------------------------------------------------- *
      open(12,file  = fn(1:fnLength),
     &        status= 'UNKNOWN',
     &        form  = 'formatted')

* --- write all nonvanishing elements of specified range in the file
      NVal = 0
      do i=iMin, iMax
      do j=jMin, jMax
        if (fVal(i,j).ne.0.0) then
          NVal = NVal + 1
          write(12,'(2(i5,1x),f20.10)') i,j,fVal(i,j)
        end if
      end do
      end do

* --- terminate writing in the file ---------------------------------- *
      close(12)
      write(*,'(i8,a)')  NVal, ' list elements written in file.'

* --- end segment ---------------------------------------------------- *
      return
      end

************************************************************************  
* === write 2d gnuplot file ========================================== *
      subroutine gnuplotList2d(fn, fnLength,
     &           fVal, iMin, iMax, iMinP, iMaxP,
     &                 jMin, jMax, jMinP, jMaxP,
     &           fFactor)
      implicit none
      character*120 fn                   ! file name of 2d list
      integer       fnLength             ! length of file name
      integer iMin,  iMax, iMinP, iMaxP  ! actual & physical index range
      integer jMin,  jMax, jMinP, jMaxP
      real*8  fVal(iMinP:iMaxP,jMinP:jMaxP)      ! 2d list
      real*8  fFactor, ffFactor       ! factor with wish input is scaled
      integer i, j,     NVal          ! counters for 2d list

* --- begin segment -------------------------------------------------- *
      write(*,'(3a)') 'Writing a 2d list in gnuplot file ',
     &                 fn(1:fnLength),'...'

* --- open the file -------------------------------------------------- *
      open(12,file  = fn(1:fnLength),
     &        status= 'UNKNOWN',
     &        form  = 'formatted')

* --- write all nonvanishing elements of specified range in the file
      NVal = 0
      do j=jMin, jMax
        do i=iMin, iMax
            NVal = NVal + 1
            write(12,'(f20.10)') fVal(i,j)
        end do
        write(12,*)
      end do

* --- terminate writing in the file ---------------------------------- *
      close(12)
      write(*,'(i8,a)')  NVal, ' list elements written in file.'

* --- end segment ---------------------------------------------------- *
      return
      end
